/* ************************************************************************** *
 *      Copyright © IPSL/LSCE, XMLIOServer, Avril 2010 - Octobre 2011         *
 * ************************************************************************** */

#ifndef __MPI_INTERFACE_HPP__
#define __MPI_INTERFACE_HPP__

/**
 * \file    mpi_interface.hpp
 * \brief   Gestion des communications MPI via une surcouche interne (entête).
 * \author  Hervé Ozdoba
 * \version 0.4
 * \date    28 Juin 2011
 */

#ifndef __XIOS_NO_EXTERN

// M(essage) P(assing) I(nterface) headers
#include <mpi.h>

// C++ standard headers
#include <vector>

#endif //__XIOS_NO_EXTERN

// XMLIOServer headers
#include "xmlioserver_spl.hpp"

// ////////////////////////////// Déclarations ///////////////////////////// //

namespace xmlioserver
{
   /// \brief Espace de nommage pour les communications via la bibliothèque MPI.
   namespace comm
   {
      typedef MPI_Fint     MPI_F_Comm; /*!< \brief Identifiant de communicateur MPI (Fortran).   */
      typedef MPI_Fint    MPI_F_Group; /*!< \brief Identifiant de groupe MPI (Fortran).          */
      typedef MPI_Fint  MPI_F_Request; /*!< \brief Identifiant de requête MPI (Fortran).         */
      typedef MPI_Fint  *MPI_F_Status; /*!< \brief Identifiant de statut MPI (Fortran).          */
      typedef MPI_Fint MPI_F_DataType; /*!< \brief Identifiant de type de données MPI (Fortran). */
      
      class CMPIManager
      {
         public : // Initialisation & Finalisation
         
            static void Initialise(int * argc, char *** argv);
            static void Finalize(void);

         public : // Communicateurs
         
            static int GetCommRank(MPI_Comm comm = CMPIManager::GetCommWorld());
            static int GetCommSize(MPI_Comm comm = CMPIManager::GetCommWorld());
            static MPI_Comm CreateComm(MPI_Group group, MPI_Comm pcomm = CMPIManager::GetCommWorld());
            static MPI_Comm GetCommWorld(void);

         public : // Autre
         
            static void Barrier(MPI_Comm comm = CMPIManager::GetCommWorld());

         public : // Groupes
         
            static MPI_Group GetGroupWorld(void);
            static MPI_Group CreateSubGroup(MPI_Group pgroup, const std::vector<int> & ranks);
            static MPI_Group CreateSubGroup(MPI_Group pgroup, int min_rank, int max_rank, int intval = 1);

         public : // Tests 
         
            static bool IsMaster(MPI_Comm comm = CMPIManager::GetCommWorld());
            static bool IsRank(int rank, MPI_Comm comm = CMPIManager::GetCommWorld());

         public : // Communication simple
         
            static void Send (MPI_Comm comm, int dest_rank, char * data,
                              std::size_t size, MPI_Request & request);
            static void Wait (MPI_Request & request);
            static bool Test (MPI_Request & request);

            static bool HasReceivedData(MPI_Comm comm, int src_rank);
            static std::size_t GetReceivedDataSize(MPI_Comm comm, int src_rank);
            static void Receive(MPI_Comm comm, int src_rank, char * data);

         public : // Communication 'complexe'
         
            // static void SendLinearBuffer(MPIComm comm, int dest_rank, CLinearBuffer & buff, MPIRequest & request);
            // static void ReceiveLinearBuffer(MPIComm comm, int src_rank, CLinearBuffer & buff);
            // static boost::shared_ptr<CLinearBuffer> ReceiveLinearBuffer(MPIComm comm, int src_rank);
            // static void ReceiveCircularBuffer(MPIComm comm, int src_rank, CCircularBuffer & buff);

         public : // Mémoire (non fonctionnel ....)
         
            static void AllocMemory(void * data, std::size_t size);
            static void FreeMemory (void * data);
         
      }; // class CMPIManager
      
   } // namespace comm
} // namespace xmlioserver

#endif //__MPI_INTERFACE_HPP__
